# gitlab-sso-auth2
本工程项目 ，主要用于针对基地AUAP统一认证中心做集成认证，并提供标准的OAUTH2对接协议，实现于gitlab的认证集成实现

## api接口说明
### GET /oauth2/authorize
> 此接口用于进行授权认证请求，获取AUAP的统一认证中心认证用户后，生成对应的授权码，重定向到gitlab的Generic OAuth2客户端进行认证
- 请求格式：

### POST /oauth2/token
> 此接口用于进行通过授权码code，生成access_token，返回OAUTH2标准的响应格式

- 请求格式（form-data）：

| 参数中文名称 | 参数名     | 类型定义   | 必填 | 说明                                                                        |
| ------ | ------- | ------ | -- | ------------------------------------------------------------------------- |
| 应用ID   | client_id   | String | 是  | 平台提供  |
| 应用秘钥 | client_secret    | String | 是  | 平台提供 |
| 应用地址 | redirect_uri   | String | 是  | 应用认证callback地址。|
| 授权code    | code | String | 是  | 通过/oauth2/authorize获取 |

- 响应格式：
```json
{
	"access_token": "",
	"refresh_token": "",
	"scope": null,
	"id_token": null,
	"token_type": "Bearer",
	"expires_in": 1713872479809
}
```
### GET userinfo
> 此接口用于通过access_token获取认证用户信息，并返回认证用户信息给gitlab

- 请求格式（header）：

| 参数中文名称 | 参数名     | 类型定义   | 必填 | 说明                                                                        |
| ------ | ------- | ------ | -- | ------------------------------------------------------------------------- |
| 授权参数   | Authorization   | String | 是  | Bearer {{access_token}}  |

- 响应格式：
```json
{
	"code": "200",
	"msg": null,
	"data": {
		"userId": "1",
		"orgName": "",
		"realname": "",
		"enName": "",
		"cardNum": "",
		"email": "test@qq.com",
		"username": "test"
	},
	"ts": 1713843759731,
	"success": true
}
```

## gitlab集成Generic OAuth2客户端
### 客户端gitlab的配置，主要修改 /etc/gitlab/gitlab.rb
配置的含义可以看gitlab官方解释，具体配置app\_id和app\_secret 自己和服务端关联起来就好，三个接口可以自己定义，gitlab有默认接口。

-   官方接口文档
    <https://docs.gitlab.com/ee/integration/omniauth.html?tab=Linux+package+%28Omnibus%29>
    <https://docs.gitlab.com/ee/integration/oauth2_generic.html>
-   参考博文
    <https://gitlab.cn/blog/2022/03/03/jihu-sso/>
    <https://blog.csdn.net/qq_16077549/article/details/118720698?share_token=35f09782-6330-4714-b52d-bcbfbddc1111>
    <https://blog.csdn.net/qq_41528213/article/details/105205564>

### 修改/etc/gitlab/gitlab.rb
> 注意:在配置过程中，需要注意id_path的配置，这里需要逐层获取对应的属性，不然会出现认证成功后，一直登录的是第一次创建的用户信息

```
#OAuth2.0
    gitlab_rails['omniauth_enabled'] = true
    # 创建一个GitLab帐户，然后通过配置文件设置将其连接到您的OmniAuth提供者帐户。值需要和omniauth_providers参数中的name值保持一致
    gitlab_rails['omniauth_allow_single_sign_on'] = ['oauth2_generic']
    # 值为false：使用OAuth登录的用户无需管理员审批，自动创建GitLab用户，值为true：使用OAuth登录自动创建的自动创建GitLab用户需要管理员审批
    gitlab_rails['omniauth_block_auto_created_users'] = true
    # 值为true：gitlab未登录的情况访问，gitlab会自动跳转到oauth2_generic对应的认证登录界面，如果要访问默认的登录界面，需在gitlab的默认登录页面地址后添加?auto_sign_in=false参数，示例：http://192.168.1.126:9121/users/sign_in?auto_sign_in=false
    gitlab_rails['omniauth_auto_sign_in_with_provider'] = 'oauth2_generic'
    gitlab_rails['omniauth_auto_link_user'] = ["oauth2_generic"]
    gitlab_rails['omniauth_auto_link_oauth2_generic_user'] = true
    gitlab_rails['omniauth_providers'] = [
      {
        'name' => 'oauth2_generic',
        # 显示在登录界面上的认证名称
        'label' => '中科知用统一认证中心',
        # 认证服务器创建的应用ID
        'app_id' => '1759151517098573824',
        # 认证服务器创建的应用的秘钥
        'app_secret' => 'ef75da89794e424d9936df3573d8b0c7',
        'args' => {
          client_options: {
            # OAuth SSO 登录认证URL
            'site' => 'http://scjoyedu.eicp.net:58000',
            # OAuth 各服务的URL
            'authorize_url' => '/auth/oauth2/authorize',
            'token_url' => '/auth/oauth2/token',
            'user_info_url' => '/auth/userinfo'
          },
          # 携带oauth2授权认证的scope的参数
          authorize_params: {
            scope: "openid"
          },
          # 定义user_info_url获取到认证用户信息的结构体
          user_response_structure: {
            # root_path用于逐层解析用户信息的Json，{data:{userId:'',realname:'',email:'',enName:''}}
            root_path: ["data"],
            # id_path是相对于root_path节点下的某个属性，作为GitLab用户的唯一id，这里配置的含义就是读取用户信息的Json中data属性下的email属性作为唯一id，对应gitlab用户的Identifier
            id_path: ["data","email"],
            # attributes是将root_path节点下的各个属性映射为标准Omniauth的用户属性，具体见 https://github.com/omniauth/omniauth/wiki/auth-hash-schema#schema-10-and-later
            attributes: {
              nickname: 'userId',
              name: 'realname',
              email: 'email',
              username:'enName'
            }
          },
          strategy_class: "OmniAuth::Strategies::OAuth2Generic"
        }
      }
    ]
```

### 更新gitlab配置和重启gitlab
```
# 修改了/etc/gitlab/gitlab.rb文件，需要执行以下命令后，配置才会生效
gitlab-ctl reconfigure

# 重启gitlab
gitlab-ctl restart
```