package com.gitlab.enums;

import com.gitlab.exception.ErrorCode;
import lombok.Getter;

/**
* @Description BasicErrorCode 类（或接口）
* @Author huangshun
* @Date 2023/6/17
*/
@Getter
public enum BasicErrorCode implements ErrorCode {

    // ============400系列============
    BAD_REQUEST("400", "请求的数据格式不符!"),
    NOT_AUTH("401", "登录凭证过期!"),
    NOT_PERMISSION("403", "抱歉，你无权限访问!"),
    NOT_FOUND("404", "请求的资源找不到!"),

    // ============500系列============
    SERVER_ERROR("500", "服务器内部错误!"),
    SERVICE_UNAVAILABLE("503", "服务器正忙，请稍后再试!"),

    // ============token错误3000-3999============
    TOKEN_INVALID("3002", "无效的登录凭证"),
    TOKEN_SIGNATURE_INVALID("3003", "无效的签名"),
    TOKEN_EXPIRED("3004", "登录凭证已过期"),
    TOKEN_MISSION("3005", "登录凭证缺失"),
    TOKEN_REFRESH_INVALID("3006", "Token刷新无效"),

    // ============错误类型E，错误来源于当前系统，用于错误问题反馈，预留编码范围：4000-6000============
    UNKNOWN_ERROR("E5000", "服务器出了点小差"),
    PARAM_ERROR("E4001", "参数错误"),
    NULL_POINTER("E4002", "空指针错误"),
    HTTP_ERROR("E4003", "HTTP错误")

    // ============警告类型W，错误来源于当前系统，用于问题警告，预留编码范围：4000-6000============

    // ============提醒类型N，错误来源于当前系统，用于业务提醒，预留编码范围：4000-6000============


    ;

    private String code;

    private String message;

    BasicErrorCode(String code, String message) {
        this.code = code;
        this.message = message;
    }
}
