package com.gitlab.exception;

import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.Objects;

/**
 * @Description BaseException 类（或接口）
 * @Author huangshun
 * @Date 2023/6/17
 */
@EqualsAndHashCode(callSuper = true)
@Data
public abstract class BaseException extends RuntimeException {
    private static final long serialVersionUID = 1L;

    /**
     * 状态码
     */
    private ErrorCode errorCode;
    /**
     * 方法名称
     */
    private String method;

    public BaseException() {

    }

    public BaseException(String message, Throwable e) {
        super(message, e);
    }

    public BaseException(ErrorCode errorCode) {
        super(errorCode.getMessage());
        this.errorCode = errorCode;
    }

    public BaseException(Throwable cause) {
        super(cause);
    }

    public BaseException(String message) {
        super(message);
    }

    public BaseException(ErrorCode errorCode, String message) {
        super(message);
        this.errorCode = errorCode;
    }

    public BaseException(ErrorCode errorCode, String message, String method) {
        super(message);
        this.errorCode = errorCode;
        this.method = method;
    }

    public ErrorCode getErrorCode() {
        return errorCode;
    }

    public void setErrorCode(ErrorCode errorCode) {
        this.errorCode = errorCode;
    }

    public String getCode() {
        return Objects.nonNull(this.errorCode) ? this.errorCode.getCode() : null;
    }
}
