package com.gitlab.util;

import cn.hutool.jwt.JWT;

import java.nio.charset.StandardCharsets;
import java.util.function.Function;

/**
* @Description JWTUtils 类（或接口）
* @Author huangshun
* @Date 2024/4/22
*/
public class JWTUtils {

    /**
     * 生成token
     * @param sk
     * @param sub
     * @param data
     * @param expiredAt
     * @return
     */
    public static String create(String sk,String sub, String data,Long expiredAt) {
        try {
            byte[] key = sk.getBytes(StandardCharsets.UTF_8);
            return JWT.create()
                    .setKey(key)
                    .setPayload("sub",sub)
                    .setPayload("data",data)
                    .setPayload("exp", expiredAt)
                    .sign();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 验证token
     * @param token
     * @param sk
     * @return
     */
    public static Boolean verify(String token,String sk) {

        return JWT.of(token).setKey(sk.getBytes(StandardCharsets.UTF_8)).verify();
    }

    /**
     * 获取数据
     * @param token
     * @param getPayload
     * @param <T>
     * @return
     */
    public static <T> T of(String token, Function<JWT,T> getPayload) {
        JWT jwt = JWT.of(token);
        return getPayload.apply(jwt);
    }

}
