package com.gitlab.util;

import com.gitlab.config.OAuth2Properties;
import lombok.extern.slf4j.Slf4j;

import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Map;

/**
 * @Description OAuth2SecurityUtils 类（或接口）
 * @Author huangshun
 * @Date 2024/4/22
 */
@Slf4j
public class OAuth2SecurityUtils {

    private OAuth2Properties properties;

    public OAuth2SecurityUtils(OAuth2Properties properties) {
        this.properties = properties;
    }

    /**
     * 公钥加密
     *
     * @param str
     * @return
     * @throws Exception
     */
    public String encryptByPublicKey(String str) throws Exception {
        String md5 = MD5Utils.MD5Encoder(this.properties.getKey() + str, "UTF-8");
        byte[] data = (str + md5).getBytes();
        //公钥加密
        byte[] encodedData = RSAUtils.encryptByPublicKey(data, this.properties.getSinglePublicKey());
        String yonghuKey = URLEncoder.encode(Base64Utils.encode(encodedData), "UTF-8");
        return yonghuKey;
    }

    /**
     * 私钥解密
     *
     * @param ecode
     * @return
     * @throws Exception
     */
    public String decryptByPrivateKey(String ecode) throws Exception {
        // 第一步：先进行URL解码
        String urlDecoderData = URLDecoder.decode(ecode, "UTF-8");
        // 第二步：进行Base64解码
        byte[] encryptedData = Base64Utils.decode(urlDecoderData);
        // 第三步：私钥解密
        byte[] decoderData = RSAUtils.decryptByPrivateKey(encryptedData, this.properties.getSinglePrivateKey());
        // 第四步：将解密的字节数组转换成字符串
        String data = new String(decoderData, "UTF-8");
        // 第五步：减去MD5的固定长度32，得到原始数据
        String str = data.substring(0, data.length() - 32);
        return str;
    }

    /**
     * 生成公钥和私钥的
     */
    private void genKeyPair() throws Exception {
        Map<String, Object> keymap = RSAUtils.genKeyPair();
        // com.ufgov.singlePublicKey
        String singlePublicKey = RSAUtils.getPublicKey(keymap);
        // com.ufgov.singlePrivateKey
        String singlePrivateKey = RSAUtils.getPrivateKey(keymap);
        System.out.println("oauth2.key=" + this.properties.getKey());
        System.out.println("oauth2.single-public-key=" + singlePublicKey);
        System.out.println("oauth2.single-private-key=" + singlePrivateKey);
    }
}
